#include "context.h"

#include "json/layout.h"
#include "json/json.h"
#include "util/format.h"

//#include "model/server.h"

#include "collections/array.h"
#include "collections/item_type/string.h"

#include <stdbool.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

static const char* langs[] = {
	"und",
	"en",
	"de",
	"fr",
	"pt",
	"ja",
};

const char* string_for_language( int lang )
{
	if( lang == 0 ) { return NULL; }
	if( lang == clang_unknown ) { return NULL; }
	if( lang - 1 >= sizeof(langs)/sizeof(langs[0]) ) { return NULL; }

	return langs[ lang - 1 ];
}

static bool context_eat( struct json_pull_parser* jpp, int* save )
{
	json_pull_parser_read_value(jpp,NULL);

	char* key;
	while( (key = json_pull_parser_read_object_key(jpp)) ) {
		free(key);
		json_pull_parser_read_value(jpp,NULL);
	}
	if( !json_pull_parser_end_object(jpp,save) ) { return false; }

	return true;
}

static bool context_read_language( struct json_pull_parser* jpp, struct ap_activity_context* ctx, int* save )
{
	char* lang = json_pull_parser_read_string(jpp);
	if( !lang ) {
		return false;
	}
	ctx->language = clang_unknown;
	for( int i = 0; i < sizeof(langs)/sizeof(langs[0]); ++i ) {
		if( 0 == strcmp(langs[i],lang) ) {
			ctx->language = i+1;
		}
	}
	if( ctx->language == clang_unknown ) {
		printf( "unknown language: %s\n", lang );
		return false;
	}
	free(lang);

	if( !json_pull_parser_end_object(jpp,save) ) {
		printf( "Object failed\n" );
		return false;
	}

	return true;
}

static bool context_reader( struct json_pull_parser* jpp, void* field_data, struct json_reflection* json_field_data, int offset )
{
	struct ap_activity_context* ctx = field_data;

	char* str = json_pull_parser_read_string(jpp);
	if( str ) {
		array_append( &ctx->extra, sizeof(str), &str );
		return true;
	}

	int save;
	if( !json_pull_parser_begin_array(jpp,&save) ) {
		return false;
	}

	while( !json_pull_parser_end_array( jpp, &save ) ) {
		str = json_pull_parser_read_string(jpp);
		if( str ) {
			array_append( &ctx->extra, sizeof(str), &str );
		} else {
			int save2;
			if( !json_pull_parser_begin_object( jpp, &save2 ) ) {
				return false;
			}
			char* key = json_pull_parser_read_object_key(jpp);
			if( 0 == strcmp("@language",key) ) {
				if( !context_read_language( jpp, ctx, &save2 ) ) { return false; }
			} else {
				if( !context_eat( jpp, &save2 ) ) { return false; }
			}
			free(key);
		}
	}

	return true;
}

static bool context_writer( struct json_writer* jw, const char* field_name, void* field_data, struct json_reflection* json_field_data, int offset )
{
	struct ap_activity_context* ctx = field_data;

	int items = 0;
	if( ctx ) {
		items += ctx->extra.count + (ctx->language ? 1 : 0 );
	}
	if( items > 0 ) {
		json_write_field_name( jw, field_name );
		if( items > 1 ) {
			fprintf( jw->f, "[" );
		}

		bool first = true;

		for( int i = 0; i < ctx->extra.count; ++i ) {
			if( !first ) { fprintf( jw->f, "," ); }
			first = false;
			json_write_string( jw->f, ctx->extra.items[i] );
		}
		const char* lang_string = string_for_language( ctx->language );
		if( lang_string ) {
			if( !first ) { fprintf( jw->f, "," ); }
			first = false;
			fprintf( jw->f, "{\"@language\":\"%s\"}", lang_string );
		}

		if( items > 1 ) {
			fprintf( jw->f, "]" );
		}
	}

	return items > 0;
}

struct json_field_type ap_activity_context_type = {
	.reader = context_reader,
	.writer = context_writer,
	.size = sizeof(struct ap_activity_context),
	.type_string = "JSON-LD Context",
};

